@@ -46,11 +46,11 @@ Example::
 """
 from ...utils.importlib import require_modules
 
-required_modules = ['falcon']
+required_modules = ["falcon"]
 
 with require_modules(required_modules) as missing_modules:
     if not missing_modules:
         from .middleware import TraceMiddleware
         from .patch import patch
 
-        __all__ = ['TraceMiddleware', 'patch']
+        __all__ = ["TraceMiddleware", "patch"]
